/*
 * @Company: hugeInfo
 * @Author: lwh
 * @Date: 2022-02-23
 * @LastEditTime: 2022-03-18 16:25:30
 * @LastEditors: ldh
 * @Version: 1.0.0
 * @Description: 功能管理-公共功能
 */
import React, { useState } from 'react';
import PropTypes from 'prop-types';
import { Table, Input, Row, Col, Button, Popconfirm, Form, Typography, Spin } from 'antd';
import { PlusOutlined } from '@ant-design/icons';
//渲染表格输入框
const EditableCell = ({ editing, dataIndex, title, inputType, record, index, children, ...restProps }) => {
	return (
		<td {...restProps}>
			{editing ? (
				<Form.Item
					name={dataIndex}
					style={{
						margin: 0,
					}}
					rules={[
						{
							required: true,
							message: `请输入${title}`,
						},
						{
							pattern: /^[^\s]*$/,
							message: '禁止输入空格',
						},
					]}
				>
					<Input placeholder={`请输入${title}`} />
				</Form.Item>
			) : (
				children
			)}
		</td>
	);
};

const EditTableView = ({
	EditTableList,
	EditTableData,
	EditTableId,
	loading,
	disabled,
	saveTableData = (data) => {},
	saveContinueTableData = (data) => {},
	removeTableData = (data) => {},
}) => {
	const [form] = Form.useForm();
	const [data, setData] = useState(EditTableList);
	const [editingKey, setEditingKey] = useState(EditTableId);
	const [addDisabled, setAddDisabled] = useState(disabled);

	const isEditing = (record) => record.id === editingKey;

	//编辑
	const edit = async (record) => {
		form.setFieldsValue({
			name: '',
			tag: '',
			des: '',
			...record,
		});
		setEditingKey(record.id);
	};

	//删除
	const handleDelete = async (id) => {
		removeTableData(id);
	};

	//取消
	const cancel = async (id) => {
		setEditingKey('');
		const row = await form.validateFields();

		const testData = data.find((i) => i.id == id);
		if (!testData.name || !testData.tag || !testData.des) {
			setData(data.filter((item) => item.id !== id));
		}
		setAddDisabled(false);
	};

	// 提交
	const save = async (id) => {
		const row = await form.validateFields();
		const tableData = form.getFieldValue();
		saveTableData({ ...tableData, appType: '2' });
	};

	// 提交并继续
	const saveContinue = async (id) => {
		const row = await form.validateFields();
		const tableData = form.getFieldValue();
		saveContinueTableData({ ...tableData, appType: '2' });
	};

	// 新增
	const addModal = () => {
		const key = String(data.length + 1);
		const newData = {
			id: key,
			name: '',
			tag: '',
			des: '',
		};
		form.setFieldsValue({
			name: '',
			tag: '',
			des: '',
		});
		setData([...data, newData]);
		setEditingKey(key);
		setAddDisabled(true);
	};

	const columns = [
		{
			title: '功能名称',
			dataIndex: 'name',
			width: '15%',
			editable: true,
		},
		{
			title: '功能标签',
			dataIndex: 'tag',
			width: '15%',
			editable: true,
		},
		{
			title: '功能说明',
			dataIndex: 'des',
			width: '15%',
			editable: true,
		},
		{
			title: '操作',
			dataIndex: 'operation',
			render: (_, record) => {
				const editable = isEditing(record);
				return editable ? (
					<span>
						<Typography.Link onClick={() => save(record.id)} style={{ marginRight: 8 }}>
							提交
						</Typography.Link>
						&nbsp;&nbsp;
						<Typography.Link onClick={() => saveContinue(record.id)} style={{ marginRight: 8 }}>
							提交并继续
						</Typography.Link>
						&nbsp;&nbsp;
						<Popconfirm title="确定取消输入?" onConfirm={() => cancel(record.id)}>
							<a>取消</a>
						</Popconfirm>
					</span>
				) : (
					<span>
						<Typography.Link disabled={editingKey !== ''} onClick={() => edit(record)}>
							修改
						</Typography.Link>
						&nbsp;&nbsp;
						<Popconfirm title="确定删除?" onConfirm={() => handleDelete(record.id)}>
							<a disabled={editingKey !== ''}>删除</a>
						</Popconfirm>
					</span>
				);
			},
		},
	];
	const mergedColumns = columns.map((col) => {
		if (!col.editable) {
			return col;
		}

		return {
			...col,
			onCell: (record) => ({
				record,
				inputType: col.dataIndex === 'age' ? 'number' : 'text',
				dataIndex: col.dataIndex,
				title: col.title,
				editing: isEditing(record),
			}),
		};
	});
	return (
		<React.Fragment>
			<Spin spinning={loading}>
				<Row className="feature-public-row">
					<Col>
						<span className="feature-public-title">公共功能</span>
					</Col>
					<Col>
						<Button type="primary" disabled={addDisabled} onClick={addModal} icon={<PlusOutlined />}>
							新增
						</Button>
					</Col>
				</Row>
				<Form form={form} component={false}>
					<Table
						pagination={false}
						components={{
							body: {
								cell: EditableCell,
							},
						}}
						bordered
						dataSource={data}
						columns={mergedColumns}
					/>
				</Form>
			</Spin>
		</React.Fragment>
	);
};

EditTableView.propTypes = {
	EditTableList: PropTypes.array,
	EditTableData: PropTypes.object,
	EditTableId: PropTypes.string,
	loading: PropTypes.bool,
};
export default EditTableView;
